
module DDR1_test #(
    parameter ADDR_WIDTH     = 24,
    parameter APP_DATA_WIDTH = 32,
    parameter APP_MASK_WIDTH = 4,
    parameter BURST_MODE     = "2"  //"2"/"4"/"8"
    )
    (
    //input
    clk, 
    rst, 
    app_rdy, 
    app_wdf_rdy,
    app_rd_data_valid,
    app_rd_data_end,
    app_rd_data, 
    init_calib_complete,
    //output
    app_en, 
    app_cmd, 
    app_addr, 
    app_wdf_data,
    app_wdf_wren,
    app_wdf_end, 
    app_wdf_mask, 
    error,
    error_flag
    );

    input clk; 
    input rst; 
    input app_rdy; 
    input app_wdf_rdy;
    input app_rd_data_valid; 
    input app_rd_data_end;
    input [APP_DATA_WIDTH-1:0] app_rd_data;
    input init_calib_complete;

    output reg app_en;
    output reg [2:0] app_cmd;
    output reg [ADDR_WIDTH-1:0] app_addr;
    output reg [APP_DATA_WIDTH-1:0] app_wdf_data;
    output reg app_wdf_wren;
    output reg app_wdf_end;
    output reg [APP_MASK_WIDTH-1:0] app_wdf_mask;
    output  error;
    output error_flag;
    
    
    localparam BURST_NUM  = BURST_MODE == "2" ? 1 :
                            BURST_MODE == "4" ? 2 :
                            BURST_MODE == "8" ? 4 : 1;
    localparam WR_TIMES   = 'd1;
    localparam CMD_CMD    = 'd65;//cmd to cmd delay
    localparam ADDR_STEP  = BURST_NUM*2;
/////////////////////////////////////////////////
reg [4:0]  data_index;
wire[4:0]  mem_wi;
reg [4:0]  mem_ri;

reg [6:0]  stopw;
reg [6:0]  stopr;
reg [8:0]  BL_WCNT;
reg [8:0]  BL_RCNT;
reg [20:0] reg_waddr;
reg [20:0] reg_raddr;
reg        cyc_flag;

reg [1:0] error_d;
reg [1:0] error_flag_d;

always@(posedge clk or posedge rst)
 if(rst)
  cyc_flag <= 1'b0;
 else if(stopr == WR_TIMES)
  cyc_flag <= 1'b1;
 else
  cyc_flag <= 1'b0;     

 always@(posedge clk or posedge rst)
    begin
      if(rst)
        begin
        BL_WCNT <= 'b0;
        stopw   <= 'b0; 
        reg_waddr<= 'd0;
        end
      else if(BL_WCNT== CMD_CMD)
        begin
        BL_WCNT <= 'b0;
        stopw   <= stopw + 1'b1;
        reg_waddr<= reg_waddr + ADDR_STEP;      //'d512;
        end
      else if(app_rdy && init_calib_complete && (stopw !== WR_TIMES) && app_wdf_rdy)
        BL_WCNT <= BL_WCNT + 1'b1;
      else if(cyc_flag)
        stopw   <= 'b0;
    end 

always@(posedge clk or posedge rst)
    begin
      if(rst)
        begin
        BL_RCNT <= 'b0;
        stopr <= 'b0;
        reg_raddr<= 'b0000_0000_0000_0000_0000;
        end
      else if( BL_RCNT == CMD_CMD )
        begin
        BL_RCNT <= 'd0;
        stopr <= stopr + 1'b1;
        reg_raddr<= reg_raddr + ADDR_STEP;        //'d512;
        end
      else if( app_rdy && init_calib_complete && (stopr !== WR_TIMES)  &&(stopw == WR_TIMES))
        BL_RCNT <= BL_RCNT + 1'b1;
      else if(cyc_flag)
        stopr <= 'b0;
    end

    wire  [APP_DATA_WIDTH-1:0] mem_data   [0:31];
    assign  mem_data[0]  = 32'h8808_7728;
    assign  mem_data[1]  = 32'h1505_5a21;
    assign  mem_data[2]  = 32'h2404_0bf2;
    assign  mem_data[3]  = 32'h3303_45e3;
    assign  mem_data[4]  = 32'h42e2_6694;
    assign  mem_data[5]  = 32'h5101_6735;
    assign  mem_data[6]  = 32'h6000_0826;
    assign  mem_data[7]  = 32'h7606_8947;

    assign  mem_data[8]  = 32'h2180_f290;
    assign  mem_data[9]  = 32'h429e_d4a1;
    assign  mem_data[10] = 32'h63ac_b6b2;
    assign  mem_data[11] = 32'h84ba_98c3;
    assign  mem_data[12] = 32'ha5c8_7ad4;
    assign  mem_data[13] = 32'hc6d6_5ce5;
    assign  mem_data[14] = 32'he7e4_3ef6;
    assign  mem_data[15] = 32'h08f2_1f07;

    assign  mem_data[16] = 32'hb870_f688;
    assign  mem_data[17] = 32'h25b1_fa19;
    assign  mem_data[18] = 32'ha462_ab2a;
    assign  mem_data[19] = 32'hd173_213b;
    assign  mem_data[20] = 32'h42b4_f24c;
    assign  mem_data[21] = 32'hd355_435d;
    assign  mem_data[22] = 32'hf446_f46e;
    assign  mem_data[23] = 32'h86b7_357f;

    assign  mem_data[24] = 32'h1c42_12ce;
    assign  mem_data[25] = 32'h3a54_34df;
    assign  mem_data[26] = 32'h5a66_56ea;
    assign  mem_data[27] = 32'h7a78_78f3;
    assign  mem_data[28] = 32'h9b8a_9a02;
    assign  mem_data[29] = 32'hbb9c_bc11;
    assign  mem_data[30] = 32'hdbae_de20;
    assign  mem_data[31] = 32'hfcb0_f03d;

assign mem_wi = reg_waddr[5:1] + data_index;

generate
    if(BURST_MODE == "2") begin: app_burst_2
        always@(posedge clk or posedge rst)
        begin
        if(rst)
            begin
                        app_en        <= 1'b0;
                        app_cmd       <= 3'b000;
                        app_wdf_wren  <= 1'b0;
                        app_wdf_end   <= 1'b0;
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_addr      <= {ADDR_WIDTH{1'b0}};
                        app_wdf_data  <= {APP_DATA_WIDTH{1'b0}};
                        data_index    <= 'b0;
            end
        else if((app_rdy ) &&(BL_WCNT== 'd1) && init_calib_complete && app_wdf_rdy)
            begin
                        app_en        <= 1'b1;
                        app_cmd       <= 3'b000;
                        app_wdf_wren  <= 1'b1;
                        app_wdf_end   <= 1'b1;
                        app_addr      <= reg_waddr;
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_wdf_data  <= mem_data[mem_wi];       
                        data_index    <= data_index + 1'b1;
            end
        else if(app_rdy &&(BL_RCNT== 'd1)   )
            begin
                        app_en        <= 1'b1;
                        app_cmd       <= 3'b001;
                        app_wdf_wren  <= 1'b0;
                        app_wdf_end   <= 1'b0;
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_addr      <= reg_raddr;
                        app_wdf_data  <= {APP_DATA_WIDTH{1'b0}};
            end
        else if(cyc_flag)
                        data_index    <= 'b0;
        else
            begin
                        app_en        <= 1'b0;
                        app_cmd       <= 3'b000;
                        app_wdf_wren  <= 1'b0;
                        app_wdf_end   <= 1'b0;
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_addr      <= {ADDR_WIDTH{1'b0}};
                        app_wdf_data  <= {APP_DATA_WIDTH{1'b0}};
            end
        end
    end
    
    else if(BURST_MODE == "4") begin: app_burst_4
        always@(posedge clk or posedge rst)
        begin
        if(rst)
            begin
                        app_en        <= 1'b0;
                        app_cmd       <= 3'b000;
                        app_wdf_wren  <= 1'b0;
                        app_wdf_end   <= 1'b0;
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_addr      <= {ADDR_WIDTH{1'b0}};
                        app_wdf_data  <= {APP_DATA_WIDTH{1'b0}};
                        data_index    <= 'b0;
            end
        else if((app_rdy ) &&(BL_WCNT== 'd1) && init_calib_complete && app_wdf_rdy)
            begin
                        app_en        <= 1'b0;
                        app_cmd       <= 3'b000;
                        app_wdf_wren  <= 1'b1;
                        app_wdf_end   <= 1'b0;
                        app_addr      <= {ADDR_WIDTH{1'b0}};
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_wdf_data  <= mem_data[mem_wi];       
                        data_index    <= data_index + 1'b1;
            end
        else if((app_rdy ) &&(BL_WCNT== 'd2) && init_calib_complete && app_wdf_rdy)
            begin
                        app_en        <= 1'b1;
                        app_cmd       <= 3'b000;
                        app_wdf_wren  <= 1'b1;
                        app_wdf_end   <= 1'b1;
                        app_addr      <= reg_waddr;
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_wdf_data  <= mem_data[mem_wi];
                        data_index    <= data_index + 1'b1;
            end
        else if(app_rdy &&(BL_RCNT== 'd1)   )
            begin
                        app_en        <= 1'b1;
                        app_cmd       <= 3'b001;
                        app_wdf_wren  <= 1'b0;
                        app_wdf_end   <= 1'b0;
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_addr      <= reg_raddr;
                        app_wdf_data  <= {APP_DATA_WIDTH{1'b0}};
            end
        else if(cyc_flag)
                        data_index    <= 'b0;
        else
            begin
                        app_en        <= 1'b0;
                        app_cmd       <= 3'b000;
                        app_wdf_wren  <= 1'b0;
                        app_wdf_end   <= 1'b0;
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_addr      <= {ADDR_WIDTH{1'b0}};
                        app_wdf_data  <= {APP_DATA_WIDTH{1'b0}};
            end
        end
    end
    
    else if(BURST_MODE == "8") begin: app_burst_8
        always@(posedge clk or posedge rst)
        begin
        if(rst)
            begin
                        app_en        <= 1'b0;
                        app_cmd       <= 3'b000;
                        app_wdf_wren  <= 1'b0;
                        app_wdf_end   <= 1'b0;
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_addr      <= {ADDR_WIDTH{1'b0}};
                        app_wdf_data  <= {APP_DATA_WIDTH{1'b0}};
                        data_index    <= 'b0;
            end
        else if((app_rdy ) &&(BL_WCNT== 'd1) && init_calib_complete && app_wdf_rdy)
            begin
                        app_en        <= 1'b0;
                        app_cmd       <= 3'b000;
                        app_wdf_wren  <= 1'b1;
                        app_wdf_end   <= 1'b0;
                        app_addr      <= {ADDR_WIDTH{1'b0}};
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_wdf_data  <= mem_data[mem_wi];       
                        data_index    <= data_index + 1'b1;
            end
        else if((app_rdy ) &&(BL_WCNT== 'd2) && init_calib_complete && app_wdf_rdy)
            begin
                        app_en        <= 1'b0;
                        app_cmd       <= 3'b000;
                        app_wdf_wren  <= 1'b1;
                        app_wdf_end   <= 1'b0;
                        app_addr      <= {ADDR_WIDTH{1'b0}};
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_wdf_data  <= mem_data[mem_wi];
                        data_index    <= data_index + 1'b1;
            end
        else if((app_rdy ) &&(BL_WCNT== 'd3) && init_calib_complete && app_wdf_rdy)
            begin
                        app_en        <= 1'b0;
                        app_cmd       <= 3'b000;
                        app_wdf_wren  <= 1'b1;
                        app_wdf_end   <= 1'b0;
                        app_addr      <= {ADDR_WIDTH{1'b0}};
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_wdf_data  <= mem_data[mem_wi];
                        data_index    <= data_index + 1'b1;
            end
        else if((app_rdy ) &&(BL_WCNT== 'd4) && init_calib_complete && app_wdf_rdy)
            begin
                        app_en        <= 1'b1;
                        app_cmd       <= 3'b000;
                        app_wdf_wren  <= 1'b1;
                        app_wdf_end   <= 1'b1;
                        app_addr      <= reg_waddr;
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_wdf_data  <= mem_data[mem_wi];
                        data_index    <= data_index + 1'b1;
            end
        else if(app_rdy &&(BL_RCNT== 'd1)   )
            begin
                        app_en        <= 1'b1;
                        app_cmd       <= 3'b001;
                        app_wdf_wren  <= 1'b0;
                        app_wdf_end   <= 1'b0;
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_addr      <= reg_raddr;
                        app_wdf_data  <= {APP_DATA_WIDTH{1'b0}};
            end
        else if(cyc_flag)
                        data_index    <= 'b0;
        else
            begin
                        app_en        <= 1'b0;
                        app_cmd       <= 3'b000;
                        app_wdf_wren  <= 1'b0;
                        app_wdf_end   <= 1'b0;
                        app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                        app_addr      <= {ADDR_WIDTH{1'b0}};
                        app_wdf_data  <= {APP_DATA_WIDTH{1'b0}};
            end
        end
    end

endgenerate

////////////////////////////////////////////////////////////
always@(posedge clk or posedge rst)
 if(rst)
    mem_ri <= 'b0;
 else if(app_rd_data_valid)
    mem_ri <= mem_ri + 1'b1;
 else
    mem_ri <= mem_ri;

always@(posedge clk or posedge rst)
 if(rst) 
   begin
    error_d[0]      <= 1'b0;
    error_flag_d[0] <= 1'b0;
   end
 else if(app_rd_data_valid & (app_rd_data != mem_data[mem_ri]))
   begin
    error_d[0]      <= 1'b1;
    error_flag_d[0] <= 1'b1;
   end
 else
   begin
    error_d[0]      <= error_d[0];
    error_flag_d[0] <= 1'b0;
   end
  
always@(posedge clk or posedge rst)
 if(rst) 
   begin
    error_d[1]      <= 1'b0;
    error_flag_d[1] <= 1'b0;
   end 
  else
   begin
    error_d[1]      <= error_d[0];
    error_flag_d[1] <= error_flag_d[0];
   end     

assign error      = error_d[1];
assign error_flag = error_flag_d[1];
 
endmodule




