

module ddr1_syn_top
(
    input           clk,//50MHz   
    input           rst_nn,//KEY1
    output [11:0] 	O_ddr_addr   ,
	output [1:0]  	O_ddr_ba     ,
	output          O_ddr_cs_n   ,
	output          O_ddr_ras_n  ,
	output          O_ddr_cas_n  ,
	output          O_ddr_we_n   ,
	output          O_ddr_clk    ,
	output          O_ddr_clk_n  ,
	output          O_ddr_cke    ,
	output [1:0] 	O_ddr_dqm    ,
	inout  [15:0] 	IO_ddr_dq    ,
	inout  [1:0] 	IO_ddr_dqs   ,
    output [4-1:0]  led  
);
    
    wire           app_rdy          ;
    wire[2:0]      app_cmd                ;
    wire           app_en             ;
    wire[23:0]     app_addr               ;
    wire           app_wdf_rdy        ;
    wire           app_wdf_wren         ;//
    wire           app_wdf_end        ;//
    wire[63:0]     app_wdf_data            ;   
    wire[7:0]      app_wdf_mask       ;   
    wire           app_rd_data_valid      ;  
    wire           app_rd_data_end        ;//unused 
    wire[63:0]     app_rd_data            ;
    
    wire           memory_clk;
    wire           pll_lock;
    
    wire           clk_x1;
    reg  [31:0]    run_cnt;
 
    wire           init_calib_complete;
    wire           error;
    wire           error_flag;
    wire           running;

//`define DEBUG_PORT_ENABLE

`ifdef DEBUG_PORT_ENABLE
    wire [6-1:0]                dbg_vector3_out; //[3*DQS_WIDTH-1:0]
    wire [4-1:0]                dbg_vector2_out; //[4*DQS_WIDTH-1:0]
    wire [7:0]                  dbg_vector1_out; 
`endif
    
    wire rst_n;

assign rst_n = rst_nn;



assign led[0] = init_calib_complete;
assign led[1] = running;
assign led[2] = error;
assign led[3] = error_flag;

//LED test
always @(posedge clk_x1 or negedge rst_n)
	if(!rst_n)
		run_cnt <= 32'd0;
	else if(run_cnt >= 32'd50_000_000)
		run_cnt <= 32'd0;
	else
		run_cnt <= run_cnt + 1'b1;

assign  running  = (run_cnt < 32'd25_000_000) ? 1'b1 : 1'b0;

DDR1_test #
    (
     .ADDR_WIDTH    (24) ,
     .APP_DATA_WIDTH(64) ,
     .APP_MASK_WIDTH(8),
     .BURST_MODE    ("4") //4,8
    )u_rd(
    .clk                (clk_x1),
    .rst                (~rst_n),    
    .app_rdy            (app_rdy),
    .app_en             (app_en),
    .app_cmd            (app_cmd),
    .app_addr           (app_addr),
    .app_wdf_data       (app_wdf_data),
    .app_wdf_wren       (app_wdf_wren),
    .app_wdf_end        (app_wdf_end),
    .app_wdf_mask       (app_wdf_mask),
    .app_wdf_rdy        (app_wdf_rdy),
    .app_rd_data_valid  (app_rd_data_valid),
    .app_rd_data_end    (app_rd_data_end),
    .app_rd_data        (app_rd_data), 
    .init_calib_complete(init_calib_complete),
    .error              (error),
    .error_flag         (error_flag)
    );

Gowin_rPLL pll(
        .clkout(memory_clk), //output clkout
        .lock(pll_lock), //output lock
        .reset(~rst_n), //input reset
        .clkin(clk) //input clkin
    );

//ddr1 ip
DDR_Memory_Interface_Top	u_DDR1_TOP
(
    .clk						(clk            ),
    .memory_clk                 (memory_clk     ),
    .pll_lock                   (pll_lock       ),
    .rst_n						(rst_n    		),
    .wr_data_en					(app_wdf_wren		),
    .wr_data_mask				(app_wdf_mask	),
    .wr_data_end				(app_wdf_end	),
    .wr_data					(app_wdf_data		),
    .cmd_en						(app_en			),
    .cmd						(app_cmd			),
    .addr						(app_addr			),
    .sr_ack						(           	),
    .ref_ack					(           	),
    .wr_data_rdy				(app_wdf_rdy	),
    .init_calib_complete		(init_calib_complete),
    .rd_data_valid				(app_rd_data_valid	),
    .rd_data_end				(app_rd_data_end	),
    .rd_data					(app_rd_data		),
    .cmd_ready					(app_rdy		),
    .sr_req						(1'b0           ),
    .ref_req					(1'b0       	),
    .burst						(1'b0   		),
    .clk_out					(clk_x1 		),
    .ddr_rst					(       		),
    `ifdef DEBUG_PORT_ENABLE
    .dbg_vector3_out            (dbg_vector3_out),
    .dbg_vector2_out            (dbg_vector2_out), 
    .dbg_vector1_out            (dbg_vector1_out), 
    `endif
    .O_ddr_addr					(O_ddr_addr		),			//output
    .O_ddr_ba					(O_ddr_ba		),			//output
    .O_ddr_cs_n					(O_ddr_cs_n		),			//output
    .O_ddr_ras_n				(O_ddr_ras_n	),			//output
    .O_ddr_cas_n				(O_ddr_cas_n	),			//output
    .O_ddr_we_n					(O_ddr_we_n		),			//output
    .O_ddr_clk					(O_ddr_clk		),			//output
    .O_ddr_clk_n				(O_ddr_clk_n	),			//output
    .O_ddr_cke					(O_ddr_cke		),			//output
    .O_ddr_dqm					(O_ddr_dqm		),			//output
    .IO_ddr_dq					(IO_ddr_dq		),			//inout
    .IO_ddr_dqs					(IO_ddr_dqs		)			//inout
); 


endmodule




