
`timescale 1ns /1ps

module tb();


   reg         clk;
   reg         rst_n;
   wire                  ddr_cs;
   wire                  ddr_ras;
   wire                  ddr_cas;
   wire                  ddr_we;
   wire                  ddr_ck;
   wire                  ddr_ck_n;
   wire                  ddr_cke;
   wire                  ddr_reset_n;
   wire [2-1:0]   		 ddr_dm;		//DM_WIDTH=2
   wire [16-1:0]   		 ddr_dq;		//DQ_WIDTH=16
   wire [2-1:0]  		 ddr_dqs;		//DQS_WIDTH=2
   wire [12-1:0]  		 ddr_addr;		//ROW_WIDTH=12
   wire [2-1:0] 		 ddr_bank;		//BANK_WIDTH=2

   wire [3:0]  led;

GSR GSR(.GSRI(1'b1));

ddr1_syn_top u_top(
  .clk(clk),
  .rst_nn(rst_n),
  .O_ddr_addr(ddr_addr),
  .O_ddr_ba(ddr_bank),
  .O_ddr_cs_n(ddr_cs),
  .O_ddr_ras_n(ddr_ras),
  .O_ddr_cas_n(ddr_cas),
  .O_ddr_we_n(ddr_we),
  .O_ddr_clk(ddr_ck),
  .O_ddr_clk_n(ddr_ck_n),
  .O_ddr_cke(ddr_cke),
  .O_ddr_dqm(ddr_dm),
  .IO_ddr_dq(ddr_dq),
  .IO_ddr_dqs(ddr_dqs),
  .led(led)
);

ddr1 ddr_model0
(
  .Clk              (ddr_ck),
  .Clk_n            (ddr_ck_n),
  .Cke              (ddr_cke),
  .Cs_n             (ddr_cs),
  .Ras_n            (ddr_ras),
  .Cas_n            (ddr_cas),
  .We_n             (ddr_we),
  .Dm               (ddr_dm[0]),
  .Ba               (ddr_bank),
  .Addr             (ddr_addr),
  .Dq               (ddr_dq[7:0]),
  .Dqs              (ddr_dqs[0])
);

ddr1 ddr_model1
(
  .Clk              (ddr_ck),
  .Clk_n            (ddr_ck_n),
  .Cke              (ddr_cke),
  .Cs_n             (ddr_cs),
  .Ras_n            (ddr_ras),
  .Cas_n            (ddr_cas),
  .We_n             (ddr_we),
  .Dm               (ddr_dm[1]),
  .Ba               (ddr_bank),
  .Addr             (ddr_addr),
  .Dq               (ddr_dq[15:8]),
  .Dqs              (ddr_dqs[1])
);

always #10 clk = ~clk;

initial begin
  $fsdbDumpfile("tb.fsdb");
  $fsdbDumpvars;
end

initial
begin
    clk = 0;
    rst_n = 1;
    
    #2400
    rst_n = 0;
    #480 
    rst_n = 1;

  #2000000;
   $finish;
end


endmodule


